# -*- coding: utf-8 -*-
## wirkungsFensterElemente.py
## © Claus Albowski

import platform
versionsNummer=platform.python_version()[0]
if versionsNummer=='3': from tkinter import *
else: from Tkinter import *

class CanvasElement(object):
    '''Alle Hauptelemente werden im Dictionary "nameUndElement" abgespeichert.
    Das Loeschen erfolgt dann ueber das tag.'''    

    @staticmethod
    def ObjektZuTag(tag):
        return CanvasElement.nameUndElement.get(tag)
    
    @staticmethod
    def NeuesElement(tag, element):
        return CanvasElement.nameUndElement.update({(tag,element)})

    @staticmethod
    def LoescheElement(zf, tag):
        element=CanvasElement.ObjektZuTag(tag)
        CanvasElement.alleFigurenIDs.remove(element.ID)
        CanvasElement.nameUndElement.pop(tag)
        zf.delete(tag)

    @staticmethod
    def LoescheAlle(zf):
        alleTags=[]
        for tag in CanvasElement.nameUndElement:
            alleTags.append(tag)
        for tag in alleTags:
            CanvasElement.LoescheElement(zf,tag)

    @staticmethod
    def IstFigurID(ID):
        return ID in CanvasElement.alleFigurenIDs
    
    @staticmethod
    def IstFigurTag(tag):
        return tag in CanvasElement.nameUndElement.keys()

    @staticmethod
    def AktualisiereWirkung(zf,von,auf,art):
        CanvasElement.LoescheElement(zf,von+'->'+auf)
        wi=WirkungElement(zf, von, auf, art)                            

    @staticmethod
    def VerschiebeAlleElemente(zf, dx, dy):
        for name in CanvasElement.nameUndElement:
            if CanvasElement.IstFigurTag(name):
                zf.move(name, dx, dy)
    
    nameUndElement=dict()
    alleFigurenIDs = []
    
    def __init__(self, zf, ID, pos, endPos, name=None):
        CanvasElement.NeuesElement(name,self)
        self.zf=zf
        self.ID=ID
        CanvasElement.alleFigurenIDs.append(ID)
        self.textID=None
        self.name=name
        self.x,self.y=pos
        self.xe,self.ye=endPos

    def GibID(self): return self.ID
    def GibTag(self): return self.name
    def GibPosition(self): return self.x,self.y
    def GibEndPosition(self): return self.xe,self.ye
    def BestimmeKoordinaten(self, pos, name):
        x,y=pos
        l=max(10+len(name)*10,40)
        xe,ye=pos[0]+l, pos[1]+30
        return x,y,xe,ye
    def Mitte(self, koordinaten):
        x,y,xe,ye=koordinaten
        return int((x+xe)/2),int((y+ye)/2)

#### ---------- FaktorElement --------------------------
class FaktorElement(CanvasElement):        

    def __init__(self, zf, name, pos=(10,10)):
        '''zeichenflaeche und eindeutigen Namen uebergeben'''
        x,y,xe,ye=self.BestimmeKoordinaten(pos, name)
        ID=zf.create_rectangle(x,y, xe,ye, width=2, tag=name)
        CanvasElement.__init__(self, zf, ID, (x,y), (xe,ye), name)
        xm,ym=self.Mitte((self.x,self.y,self.xe,self.ye))
        self.textID=zf.create_text(xm,ym, text=name, tag=name, font="Sans 12 bold")

#### ---------- WirkungElement --------------------------
class WirkungElement(CanvasElement):

    def __init__(self, zf, von, auf, art='+',bogenOrientierung=1):
        '''zeichenflaeche und eindeutige Namen uebergeben'''
        self.orBog=bogenOrientierung # Orientierung des Pfeilbogens
        self.von,self.auf,self.art=von, auf,art
        name=von+'->'+auf
        self.x,self.y=self.Mitte(zf.bbox(von))
        self.xe,self.ye=self.Mitte(zf.bbox(auf))
        xm,ym=self.Mitte((self.x,self.y,self.xe,self.ye))
        x,y = self.x+0.2*(xm-self.x),self.y+0.2*(ym-self.y)## Ueberschneidung vermeiden
        xe,ye = self.xe+0.25*(xm-self.xe),self.ye+0.25*(ym-self.ye)
        self.ve=0.25 # VersatzFaktor fuer orthogonal       
        dx,dy=self.orBog*(1+int(self.ve*(ye-y))),self.orBog*(-1-int(self.ve*(xe-x)))
        ID=zf.create_line(x,y,xm-dx,ym-dy,xe,ye,
                          arrow=LAST, smooth=True, width=2, tag=name)
        CanvasElement.__init__(self, zf, ID, (x,y), (xe,ye), name)
        xt,yt=self.PositionVonPlusOderMinus((self.x,self.y,self.xe,self.ye))
        self.kreisID=zf.create_oval(xt-10, yt-10, xt+10, yt+10, tag=name, fill='white')
        self.textID=zf.create_text(xt,yt, text=self.art, tag=name, font="Sans 12 bold")
        ## text aendern mit zf.itemconfigure(self.textID, text=...)

    def GibVon(self): return self.von
    def GibAuf(self): return self.auf
    def GibArt(self): return self.art
    def GibBogenOrientierung(self): return self.orBog

    def AendereVon(self, neuerName):
        '''erzeugt ein neues WirkungElement'''
        tag=self.von+'->'+self.auf
        self.zf.delete(tag)
        return WirkungElement(self.zf, neuerName, self.auf, self.art)

    def AendereAuf(self, neuerName):
        tag=self.von+'->'+self.auf
        self.zf.delete(tag)
        return WirkungElement(self.zf, self.von, neuerName, self.art)

    def AendereArt(self):
        if self.art=='+': self.art='-'
        else: self.art='+'
        self.zf.itemconfigure(self.textID, text=self.art)

    def PositionVonPlusOderMinus(self, koordinaten):
        x,y,xe,ye=koordinaten
        return 0.3*x+0.7*xe-0.05*self.orBog*(ye-y),0.3*y+0.7*ye+0.05*self.orBog*(xe-x)
